#!/usr/bin/env python
#coding: utf8 

import sys
import scipy
import Machine
import cPickle
import time
from numpy import *

	
#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# This script demonstrates how the machine was trained in arxiv.org/abs/1508.05315
# The script takes a data set and trains a machine on it. The dataset is stored as a pickled 
# (https://docs.python.org/2/library/pickle.html) dictionary containinng the following keys: 
# 'Co': Relative coordinates of each atom in the unicell
# 'Ce': Coordinates lattice coordinates of the unit cell (Å)
# 'N' : Number of atoms in the unit cell 
# 'T' : Formation energy of the crystal unit cell (eV)
# 'X' : Representation of the crystal
# 'Z' : Atomic charge of atoms in the unit cell 
# Each key points to a list where each element constitutes a crystal structure.
#---------------------------------------------------------------------------------------------------------------
# The script takes two input parameters, pathIn and pathOut. pathIn is the path to the data set.
# pathOut is the path where the trained dataset will be stored.
#
#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	
def main():	  
	
	pathIn = str(sys.argv[1])
	pathOut = str(sys.argv[2])
	
	##Initializing the machine
	#------------------------------------------------------
	machine = Machine.ML(pathIn,sigma = 8,oFPunish = 1E-9,
				linOfPunish = 10, ker = Machine.LapKer())
	#------------------------------------------------------
	t1 = time.clock()
	print 'Training machine'
	machine.FindAlpha()	##Training the machine
	t2 = time.clock()
	print 'Done, training took ', t2 - t1 , ' seconds'
	f = open(pathOut, 'wb')
	cPickle.dump([machine.o,machine.DistMatrix,machine.atomRegConst],f, protocol=2)
	f.close()
	
if __name__ == "__main__":
	main()
	print 'end'  
