#!/usr/bin/env python
#coding: utf8 
import itertools as itr
import sys
import scipy
import Machine
import cPickle
import time
from numpy import *

#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# This script predicts the formation energies of all maingroup elpasolites (See arxiv.org/abs/1508.05315).
#
#---------------------------------------------------------------------------------------------------------------
# The script takes two input parameters, pathMachine and pathAllStructures. pathMachine is the path to the
# machine (see TrainMachine.py). pathAllStructures is the path to where the fromation energies of the
# predicted structures will be stored. The energies are stored as a pickled (https://docs.python.org/2/library/pickle.html) 
# dictionary containing the following keys:
# 'N' : Number of atoms in the unit cell 
# 'T' : Predicted formation energy of the crystal unit cell (eV)
# 'X' : Representation of the crystal
# 'Z' : Atomic charge of atoms in the unit cell 
# Each key points to a list where each element constitutes a crystal structure.
#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	

##Periodic table possition(only s and p block)
#------------------------------------------------------
PTP = {1:[1,1],
		2:[1,8], 3:[2,1], 4:[2,2], 5:[2,3],\
		6:[2,4], 7:[2,5], 8:[2,6], 9:[2,7],\
		10:[2,8],11:[3,1],12:[3,2],13:[3,3],\
		14:[3,4],15:[3,5],16:[3,6],17:[3,7],\
                18:[3,8],19:[4,1],20:[4,2],31:[4,3],32:[4,4],\
                33:[4,5],34:[4,6],35:[4,7],36:[4,8],37:[5,1],\
                38:[5,2],49:[5,3],50:[5,4],51:[5,5],52:[5,6],\
                53:[5,7],54:[5,8],55:[6,1],56:[6,2],81:[6,3],\
                82:[6,4],83:[6,5]}
#------------------------------------------------------


def main():	  
	
	pathMachine = str(sys.argv[1])
	pathAllStructures = str(sys.argv[2])
	
	##Loading machine
	#------------------------------------------------------
	f = open(pathMachine, 'rb')
	inp = cPickle.load(f)
	f.close()
	machine = Machine.ML(inp[0],sigma = 8,oFPunish = 1E-9,
				linOfPunish = 1, ker = Machine.LapKer())
	machine.DistMatrix = inp[1]
	machine.atomRegConst = inp[2]
	#------------------------------------------------------
	
	AllElements = [1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18, 19 ,20,31,32,33,34,35,36, 37 ,38,49,50,51,52,53,54,55,56,81,82,83]##Atomic charge of all maingroup elements
	NumElementsInSite  = [1,1,2,6]##Number of elements corresponding to each site
	
	startTime = time.clock()
	print 'Preparing input'
	
	oPredict = {} ##Dictionary which will store all data
	per = asarray([list(p) for p in itr.permutations(AllElements,4)])##Generating all permutations

	oPredict['X'] = asarray([[PTP[pp] for pp in p] for p in per])##Creating representation
	
	##Creating the elements belonging to each crystal
	#------------------------------------------------------
	Z = []
	for p in per:
		z = []
		for i in range(len(p)):
			for j in range(NumElementsInSite[i]):
				z.append(p[i])
		Z.append(z)	
	oPredict['Z'] = asarray(Z)			   
	oPredict['N'] = asarray([len(z) for z in oPredict['Z']])
	#------------------------------------------------------
	##Setting creating input for atomic contribution 
	#------------------------------------------------------
	def setAtomCount(o):
		tmp = asarray([bincount(z) for z in o['Z']])			
		a = zeros(len(o['Z'])*118).reshape(len(o['Z']),118)
		for i in range(len(tmp)):
			a[i,0:tmp[i].shape[0]]= tmp[i]            
			a[i,0] = 1	   
		o['atomCount'] = a
		return o	
	oPredict = setAtomCount(oPredict)
	#------------------------------------------------------
	prepTime = time.clock()
	print 'Done, preparing took ', prepTime - startTime , ' seconds'
	print 'Calculating Energies'
	oPredict['T'] = machine.Energy_list(oPredict['X'],oPredict['atomCount'])##Predicing Energies for all structures
	calcTime = time.clock()
	print 'Done, calculating took ', calcTime - startTime , ' seconds'
	
	##Storing results
	#------------------------------------------------------
	f = open(pathAllStructures, 'wb')
	cPickle.dump(oPredict,f, protocol=2)
	f.close()
	#------------------------------------------------------
	
	
if __name__ == "__main__":
	main()
	print 'end'  
